#include "udp.h"

UDP::UDP()
{

}

/****************UDP初始化**************/
bool UDP::UDPInit(char addr)
{
    int ret = 0;
    ServerPort = 0;
    QString strPort,strAddr;
    /*******************读取配置文件来确定ServerPort(以避免修改IP/Port/Addr后无法连接的问题)*********************/
    readPort = new QSettings("./port.ini", QSettings::IniFormat);
    readPort->beginGroup("Port");
    strPort =readPort->value("port").toString();
    strAddr = readPort->value("addr").toString();
    readPort->endGroup();
    delete  readPort;
    /*******************判断配置文件是否存在*********************/
    if(strPort.isEmpty())
    {
        SPort = 1024;
    }
    else
    {
        SPort = strPort.toInt();
    }

    if(strAddr.isEmpty())
    {
        LocalAddr = addr;
    }
    else
    {
        LocalAddr = strAddr.toInt();
    }
    /*******************常用指令初始化*****************************/
    UDPDirectInit(LocalAddr);
    /*******************创建UDP句柄*****************************/
    Local = new QUdpSocket;
    /*******************获取本地并添加进QHostAddress中************/
    LocalIp = getIP();
    LocalAddress.setAddress(LocalIp);
    Local->bind(LocalAddress,LocalPort);
    /*******************以广播的方式发送命令到默认的1024端口************/
    ret = Local->writeDatagram(SendInit.data(),SendInit.size(),QHostAddress::Broadcast ,SPort);//开启广播并发送命令
    if(ret <= 0)
    {
        UDPClose();
        return false;
    }
    /*******************清理read缓存************/
    read.clear();
    /*******************延时1s后读取数据************/
    sleep(1);
    while (Local->hasPendingDatagrams())
    {
        read.resize(Local->pendingDatagramSize());
        ret = Local->readDatagram(read.data(), read.size(),&ServerAddress,&ServerPort);

    }

    if(ret <= 0)
    {
        UDPClose();
        return false;
    }
    /*******************取得下位机IP斌添加到ServerAddress中************/
    ServerIp = ServerAddress.toString();
    ServerAddress.setAddress(ServerIp);
    return true;

}

/****************获取下位机解析后的数据**************/
bool UDP::UDPData(double *Sum)
{
    int data[6] = {0};//数据解析后的中间变量值
    /*******************发送命令************/
    int ret = 0;
    ret = Local->writeDatagram(SendData.data(),SendData.size(),ServerAddress,SPort);
    if(ret <= 0) return false;
    /*******************清理接收缓存************/
    UdpData[32] = {0};
    /*******************读取数据************/
    while (Local->hasPendingDatagrams())
    {
        ret = Local->readDatagram(UdpData,21);
    }
    if(ret <= 0 )  return false;
    /*******************对读取到的数据进行转换************/
    for(int i = 0;i < 21 ; ++i) ReadData[i] = (unsigned char)UdpData[i];
    for(int i = 0;i < 6; ++i)
    {
        if (ReadData[i*3+3] & 0x80)				// 判断数据的正负号
        {
            ReadData[i*3+3] &= 0x7F;
        }
        for(int j = 0; j < 3; ++j)
        {
            int n = i*3+3+j;
            data[i] = (data[i] << 8) + ReadData[n];
        }
        sum[i] = data[i]/1000.0;
        if (UdpData[i*3+3] & 0x80)	sum[i] *= -1;
    }
    for(int i = 0; i < 6; ++i)
    {
        Sum[i] = sum[i];
    }
    return true;

}

/****************对下位机进行零点清除**************/
bool UDP::UDPClearZero()
{
    /*******************发送命令************/
    int ret = 0;
    ret = Local->writeDatagram(SendClear.data(),SendClear.size(),ServerAddress,SPort);
    if(ret <= 0) return false;
    /*******************清理read缓存************/
    read.clear();
    /*******************延时100ms后读取数据************/
    sleep(0.1);
    while (Local->hasPendingDatagrams())
    {
        read.resize(Local->pendingDatagramSize());
        ret = Local->readDatagram(read.data(), read.size());
    }
    if(ret <= 0)  return false;
    return true;
}

/****************对下位机进零点设置**************/
bool UDP::UDPSetZero()
{
    /*******************发送命令************/
    int ret = 0;
    ret = Local->writeDatagram(SendSetClear.data(),SendSetClear.size(),ServerAddress,SPort);
    if(ret <= 0) return false;
    /*******************清理read缓存************/
    read.clear();
    /*******************延时100ms后读取数据************/
    sleep(0.1);
    while (Local->hasPendingDatagrams())
    {
        read.resize(Local->pendingDatagramSize());
        ret = Local->readDatagram(read.data(), read.size());
    }
    if(ret <= 0 )  return false;
    return true;
}

/****************获取下位机的IP/Port.**************/
bool UDP::UDPSearchIpPort(QString *IP, int *Port)
{
    /*******************发送send命令************/
    int ret = 0;
    ret = Local->writeDatagram(SendSearch.data(),SendSearch.size(),ServerAddress,SPort);
    if(ret <= 0) return false;
    /*******************清理read缓存************/
    read.clear();
    /*******************延时100ms后读取数据************/
    sleep(0.1);
    while (Local->hasPendingDatagrams())
    {
        read.resize(Local->pendingDatagramSize());
        ret = Local->readDatagram(read.data(), read.size(),&ServerAddress,&ServerPort);
    }
    if(ret <= 0 )  return false;
    /*******************返回IP和Port的值************/
    *IP = ServerAddress.toString();
    *Port = ServerPort;
    return true;
}

/****************对下位机设备进行重启**************/
bool UDP::UDPRestartDevice()
{
    /*******************发送send命令************/
    int ret = 0;
    ret = Local->writeDatagram(SendRestar.data(),SendRestar.size(),ServerAddress,SPort);
    if(ret <= 0) return false;
    /*******************清理read缓存************/
    read.clear();
    /*******************延时100ms后读取数据************/
    sleep(0.1);
    while (Local->hasPendingDatagrams())
    {
        read.resize(Local->pendingDatagramSize());
        ret = Local->readDatagram(read.data(), read.size());
    }
    if(ret <= 0) return false;
    return  true;
}

/*******************设置下位机地址************/
bool UDP::UDPSetAddr(char NewAddr)
{

    /*******************把NewAddr配置文件(以避免修改SetPort后无法连接的问题)*********************/
    readPort = new QSettings("./port.ini", QSettings::IniFormat);
    readPort->beginGroup("Port");
    readPort->setValue("addr",QString::number(NewAddr,10));
    readPort->endGroup();
    delete  readPort;
    /*******************发送指令(固定指令)*****************************/
    char send[6] = {0};
    send[0] = LocalAddr;
    send[1] = 0x10;
    send[2] = 0x02;
    send[3] = 0x03;
    send[4] = 0x01;
    send[5] = NewAddr;
    QByteArray SendSetAddr;
    for(int i = 0; i < 6; ++i)
    {
        SendSetAddr[i] = send[i];
    }
    /*******************发送send命令************/
    int ret = 0;
    ret = Local->writeDatagram(SendSetAddr.data(),SendSetAddr.size(),ServerAddress,SPort);
    if(ret <= 0) return false;
    /*******************清理read缓存************/
    read.clear();
    /*******************延时10ms后读取数据************/
    sleep(0.1);
    while (Local->hasPendingDatagrams())
    {
        read.resize(Local->pendingDatagramSize());
        ret = Local->readDatagram(read.data(), read.size());
    }
    if(ret <= 0) return false;
    /*******************设置Addr成功后重启设备************/
    UDPRestartDevice();
    /*******************设置Addr成功后关闭连接************/
    UDPClose();

    return true;
}

/*******************设置下位机的IP/Port************/
bool UDP::UDPSetIPPort(QString NewIp, int NewPort)
{
    /*******************把ServerPort配置文件(以避免修改SetPort后无法连接的问题)*********************/
    readPort = new QSettings("./port.ini", QSettings::IniFormat);
    readPort->beginGroup("Port");
    readPort->setValue("port",QString::number(NewPort,10));
    readPort->setValue("IP",NewIp);
    readPort->endGroup();
    delete  readPort;


    QString str;
    int a;
    /*******************发送指令(固定指令)*****************************/
    char sendIp[9] = {0};
    sendIp[0] = LocalAddr;
    sendIp[1] = 0x10;
    sendIp[2] = 0x02;
    sendIp[3] = 0x3e;
    sendIp[4] = 0x04;
    /*******************把QString类型的IP转化为char*****************************/
    for(int i = 0;i < 4; ++i)
    {
        str = NewIp.section(".",i,i);
        a = str.toInt();
        sendIp[5+i] = a;
    }
    QByteArray SendSetIP;
    for(int i = 0 ; i< 9;++i)
    {
        SendSetIP[i] = sendIp[i];
    }
    /*******************发送sendIp命令************/
    int ret = 0;
    ret = Local->writeDatagram(SendSetIP.data(),SendSetIP.size(),ServerAddress,SPort);
    if(ret <= 0) return false;
    /*******************清理read缓存************/
    read.clear();
    /*******************延时10ms后读取数据************/
    sleep(0.01);
    while (Local->hasPendingDatagrams())
    {
        read.resize(Local->pendingDatagramSize());
        ret = Local->readDatagram(read.data(), read.size());

    }
    if(ret <= 0) return false;

    /*******************发送指令(固定指令)*****************************/
    char sendport[7] = {0};
    sendport[0] = LocalAddr;
    sendport[1] = 0x10;
    sendport[2] = 0x02;
    sendport[3] = 0x42;
    sendport[4] = 0x02;
    /******************int类型的Port转化为两个char******************************/
    sendport[5] = NewPort >>8;
    sendport[6] = (NewPort <<8) >> 8;

    QByteArray SendSetPort;
    for(int i = 0; i < 7; ++i)
    {
        SendSetPort[i] = sendport[i];
    }
    /*******************发送sendPort命令************/
    ret = Local->writeDatagram(SendSetPort.data(),SendSetPort.size(),ServerAddress,SPort);
    if(ret <= 0) return false;
    /*******************清理read缓存************/
    read.clear();
    /*******************延时10ms后读取数据************/
    sleep(0.01);
    while (Local->hasPendingDatagrams())
    {
        read.resize(Local->pendingDatagramSize());
        ret = Local->readDatagram(read.data(), read.size());
    }
    if(ret <= 0) return false;
    /*******************设置IP和Port成功后重启设备************/
    UDPRestartDevice();
    /*******************设置IP和Port成功后关闭连接************/
    UDPClose();
    return true;
}

/*******************关闭连接并释放UDP句柄************/
void UDP::UDPClose()
{
    Local->close();
    delete Local;
}

/*******************对常用指令进行初始化************/
void UDP::UDPDirectInit(char addr)
{
    /*******************初始化指令*********************/
    sendInit[0]=addr;
    sendInit[1]=0x13;
    sendInit[2]=0x00;
    sendInit[3]=0x00;
    sendInit[4]=0x01;
    for(int i = 0; i < 5; ++i)
    {
        SendInit[i] = sendInit[i];
    }
    /*******************取得数据指令*********************/
    sendData[0] = addr;
    sendData[1] = 0x03;
    sendData[2] = 0x02;
    sendData[3] = 0xD0;
    sendData[4] = 0x12;
    for(int i = 0; i < 5; ++i)
    {
        SendData[i] = sendData[i];
    }
    /*******************设备清零指令*********************/
    sendClear[0] = addr;
    sendClear[1] = 0x19;
    sendClear[2] = 0x00;
    sendClear[3] = 0x00;
    sendClear[4] = 0x01;
    for(int i = 0; i < 5; ++i)
    {
        SendClear[i] = sendClear[i];
    }

    sendSetClear[0] = addr;
    sendSetClear[1] = 0x12;
    sendSetClear[2] = 0x00;
    sendSetClear[3] = 0x00;
    sendSetClear[4] = 0x01;
    for(int i = 0; i < 5; ++i)
    {
        SendSetClear[i] = sendSetClear[i];
    }
    /*******************搜索IP/Port指令*********************/
    sendSearch[0] = addr;
    sendSearch[1] = 0x15;
    sendSearch[2] = 0x00;
    sendSearch[3] = 0x00;
    sendSearch[4] = 0x01;
    for(int i = 0; i < 5; ++i)
    {
        SendSearch[i] = sendSearch[i];
    }
    /*******************设备重启指令*********************/
    sendRestar[0] = addr;
    sendRestar[1] = 0x18;
    sendRestar[2] = 0x00;
    sendRestar[3] = 0x01;
    sendRestar[4] = 0x00;
    for(int i = 0; i < 5; ++i)
    {
        SendRestar[i] = sendRestar[i];
    }
}

/*******************获取本机IP*********************/
QString UDP::getIP()
{
    QString  str1;
    QList<QHostAddress> list = QNetworkInterface::allAddresses();
    foreach (QHostAddress address, list)
    {
        if(address.protocol() == QAbstractSocket::IPv4Protocol)
            str1 = address.toString();
        str1 = str1.section(".",0,2);
        if(str1 == "192.168.1")
            return address.toString();
    }
    return 0;
}
