/***********************************************************
* Function Name    ： UDP.
* Description      ： UDP通信.
* Development Tool ： QT Creator5.7.1
* Test Platform    ： Ubuntu 16.04 64位
* ***********************************************************/


#ifndef UDP_H
#define UDP_H
#include <QHostAddress>
#include <QUdpSocket>
#include <QNetworkInterface>
#include <QtNetwork>
#include <QString>
#include <QByteArray>
#include <QtCore/QCoreApplication>
#include <QSettings>
#include <unistd.h>

class UDP
{
public:
    UDP();//空函数
public:
    /***********************************************************
    * Function Name  ： UDPInit.
    * Description    ： UDP初始化.
    * Input          ： addr - 下位机地址.
    * Return         ：成功 —— true，失败——false.
    * ***********************************************************/
    bool UDPInit(char addr);

    /***********************************************************
    * Function Name  ： UDPData.
    * Description    ： 获取下位机解析后的数据.
    * Output         ： Sum - 解析后的数据.
    *                Sum[0] - Tx , Sum[1] - Ty , Sum[2] - Tz
    *                Sum[3] - Fx , Sum[4] - Fy , Sum[5] - Fz
    * Return         ：成功 —— true，失败——false.
    * ***********************************************************/
    bool UDPData(double *Sum);

    /***********************************************************
    * Function Name  ： UDPClearZero.
    * Description    ： 对下位机进行零点清除.
    * Return         ： 成功 —— true，失败——false.
    * ***********************************************************/
    bool UDPClearZero();

    /***********************************************************
    * Function Name  ： UDPSetZero.
    * Description    ： 对下位机进零点设置.
    * Return         ： 成功 —— true，失败——false.
    * ***********************************************************/
    bool UDPSetZero();

    /***********************************************************
    * Function Name  ： UDPSearchIpPort.
    * Description    ： 获取下位机的IP/Port.
    * Output         ： ServerIP - 下位机IP.
    *                ： SeverPort - 下位机端口号.
    * Return         ： 成功 —— true，失败——false.
    * ***********************************************************/
    bool UDPSearchIpPort(QString *ServerIP, int *SeverPort);

    /***********************************************************
    * Function Name  ： UDPRestartDevice.
    * Description    ： 对下位机设备进行重启.
    * Return         ： 成功 —— true，失败——false.
    * ***********************************************************/
    bool UDPRestartDevice();

    /***********************************************************
    * Function Name  ： UDPSetAddr.
    * Description    ： 设置下位机地址.
    * Input          ： NewAddr - 新的下位机地址
    * Return         ： 成功 —— true，失败——false.
    * ***********************************************************/
    bool UDPSetAddr(char NewAddr);

    /***********************************************************
    * Function Name  ： UDPSetIPPort.
    * Description    ： 设置下位机的IP/Port.
    * Input          ： NewIp - 新的下位机IP.
    *                ： NewPort - 新的下位机端口号.
    * Return         ： 成功 —— true，失败——false.
    * ***********************************************************/
    bool UDPSetIPPort(QString NewIp,int NewPort);

    /***********************************************************
    * Function Name  ： UDPClose.
    * Description    ： 关闭连接并释放UDP句柄.
    * ***********************************************************/
    void UDPClose();

    /***********************************************************
    * Function Name  ： UDPDirectInit.
    * Description    ： 对常用指令进行初始化.
    * Input          ： addr - 下位机地址
    * ***********************************************************/
    void UDPDirectInit(char addr);

    /***********************************************************
    * Function Name  ： getIP.
    * Description    ： 获取本机IP.
    * Return         ： 返回本机IP.
    * ***********************************************************/
    QString getIP();

private:
    QUdpSocket *Local;           //UDP句柄
    QHostAddress LocalAddress;   //保存本机网络信息
    QHostAddress ServerAddress;  //保存下位机网络信息

    QString LocalIp;             //保存本机IP;
    QString ServerIp;            //保存下位机IP;
    int LocalPort = 8080;        //保存本机端口号，默认为8080;
    char LocalAddr;              //保存下位机地址;
    QSettings *readPort;         //配置文件操作句柄;

    QByteArray read;             //读取数据缓存;
    unsigned short ServerPort;   //保存下位机IP;
    unsigned short SPort = 1024; //保存下位机机端口号，默认为1024;
    unsigned char ReadData[32];  //保存读取的下位机数据;
    char UdpData[32];            //数据解析中间变量;
    double sum[6];               //保存解析后的数据;

    char sendInit[5];            //初始化指令;
    QByteArray SendInit;

    char sendData[5];            //取得数据指令;
    QByteArray SendData;

    char  sendClear[5];          //设备清除零点指令;
    QByteArray SendClear;

    char  sendSetClear[5];       //设备设置零点指令;
    QByteArray SendSetClear;

    char sendSearch[5];          //搜索IP/Port指令;
    QByteArray SendSearch;

    char sendRestar[5];          //设备重启指令;
    QByteArray SendRestar;

};

#endif // UDP_H
